@@ -17,14 +17,14 @@ for (cleanup in c(FALSE, TRUE)) {
   print(f)
 
   resolve(f)
-
+  
   ## Assert future is listed as resolved
   stopifnot(resolved(f))
 
   reg <- f$config$reg
-
   ## Force garbage collection of future which will possibly
   ## result in the removal of batchtools registry files
+
   reg.finalizer(f, function(f) {
     message("Garbage collection future ...")
     print(f)
@@ -33,12 +33,13 @@ for (cleanup in c(FALSE, TRUE)) {
   rm(list = "f")
   gc()
   message(" - Future removed and garbage collected.")
-  mprintf(" - batchtools Registry path (%s) exists: %s",
+  mprintf(" - batchtools Registry path (%s) exists: %s\n",
           sQuote(reg$file.dir), file_test("-d", reg$file.dir))
-
+  
   ## Assert removal of files only happens if there was not
   ## a failure and option future.delete is not TRUE.
   if (!cleanup) {
+    ## FIXME: Does the new future::FutureResult trigger garbage collection?
     stopifnot(file_test("-d", reg$file.dir))
     log <- batchtools::getLog(reg = reg, id = 1L)
     print(log)
